import 'dart:convert';
import 'dart:core';
import 'package:crypto/crypto.dart';
import 'dart:typed_data';

extension StringExtraUtils on String {
  String? get first {
    return this.isEmpty ? null : this[0];
  }

  String? get last {
    return this.isEmpty ? null : this[this.length-1];
  }

  /// return a new string removing first n characters
  String removeLeft(int characters) {
    return this.substring(characters);
  }

  /// return a new string removing last n characters
  String removeRight(int characters) {
    return this.substring(0,this.length-characters);
  }

  Object get Json {
    var value = jsonDecode(this);

    if(value is! List)
      return Map<String,dynamic>.from(value);

    return value;
  }

  bool get isNullOrEmpty => this == null || this.isEmpty;
  bool get isNotNullOrEmpty => !(this == null || this.isEmpty);

  String get ToBase64 {
    var bytes = utf8.encode(this);
    return base64.encode(bytes);
  }

  Uint8List get FromBase64 {
    return base64.decode(this);
  }

  String get FromBase64ToString {
    return String.fromCharCodes(this.FromBase64);
  }

  String get ToBase64Url {
    var bytes = utf8.encode(this);
    return base64Url.encode(bytes);
  }

  Uint8List get FromBase64Url {
    return base64Url.decode(this);
  }

  String get FromBase64UrlToString {
    return String.fromCharCodes(this.FromBase64Url);
  }

  String? JWTPayload(String? secret) {
    var parts = this.split('.');

    if(parts.length != 3)
      throw 'invalid jwt token';

    if(secret == null)
      return jsonDecode(parts[1].FromBase64UrlToString);

    if(Hmac(sha512, utf8.encode(secret)).convert(utf8.encode(parts[0]+'.'+parts[1])) == parts[2])
      return jsonDecode(parts[1].FromBase64UrlToString);
    else
      throw 'invalid jwt sing';
  }
}
    